#ifndef __LXCANL2_H
#define __LXCANL2_H

/*-----------------------------------------------------------------
   LevelX general definitions
   -- CanL2

ih20010116- CanL2GetTxFifoSize added
-----------------------------------------------------------------*/

/* !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
Note that the compiler must set on byte alignment!
For MS-C++ it is:
Project Settings.C/C++.Struct member aligment = 1 Byte
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! */

/* --- CAN level 2 object identifier --- */
#define LX_OID_CanL2GetChannelInfoReq             ((unsigned char)0x02)
#define LX_OID_CanL2GetChannelInfoConf            ((unsigned char)0x03)
#define LX_OID_CanL2SetupReq                      ((unsigned char)0x04)
#define LX_OID_CanL2SetupConf                     ((unsigned char)0x05)
#define LX_OID_CanL2SetMaskAndCodeReq             ((unsigned char)0x06)
#define LX_OID_CanL2SetMaskAndCodeConf            ((unsigned char)0x07)
#define LX_OID_CanL2BusOnReq                      ((unsigned char)0x08)
#define LX_OID_CanL2BusOffReq                     ((unsigned char)0x0a)
#define LX_OID_CanL2ChipStatusChangedInd          ((unsigned char)0x0d)
#define LX_OID_CanL2ChipStatusChangedTimeStampInd ((unsigned char)0x0f)
#define LX_OID_CanL2ReadErrorCounterReq           ((unsigned char)0x10)
#define LX_OID_CanL2ReadErrorCounterConf          ((unsigned char)0x11)
#define LX_OID_CanL2ChipReceiverOverFlowInd       ((unsigned char)0x13)
#define LX_OID_CanL2TxObjectReq                   ((unsigned char)0x14)
#define LX_OID_CanL2TxObjectConf                  ((unsigned char)0x15)
#define LX_OID_CanL2TxObjectTimeStampConf         ((unsigned char)0x17)
#define LX_OID_CanL2RxObjectInd                   ((unsigned char)0x19)
#define LX_OID_CanL2RxObjectTimeStampInd          ((unsigned char)0x1b)

#define LX_OID_CanL2GetTxFifoSizeReq              ((unsigned char)0x1c)
#define LX_OID_CanL2GetTxFifoSizeConf             ((unsigned char)0x1d)

#define LX_OID_CanL2SelectPhyDrvReq               ((unsigned char)0x30)
#define LX_OID_CanL2SelectPhyDrvConf              ((unsigned char)0x31)
#define LX_OID_CanL2SetPhyDrvSwcHighVoltReq       ((unsigned char)0x32)
#define LX_OID_CanL2SetPhyDrvSwcHighVoltConf      ((unsigned char)0x33)


#ifndef mLX_ObjHead
  #define mLX_ObjHead \
    unsigned char ObjLen;\
    unsigned char GroupID;\
    unsigned char ObjectID
#endif

typedef unsigned short twCanStdId;
typedef unsigned long tdwCanXtdId;
typedef unsigned long tdwIdField;
typedef struct
{
  // all big endian, left bounded (same as tsCanObj.IdField)
  // MaskBits: 1 - must match with CodeBit, 0 - don't care
  //   If all bits of mask and code are set -> Xtd or Std are not used ->
  //   necessary from 82527 because it can receive global only one type
  twCanStdId StdMask;
  twCanStdId StdCode;
  tdwCanXtdId XtdMask;
  tdwCanXtdId XtdCode;
} tsCmdSetMaskAndCode;
#define cCmdMaskAndCode_Ok 0
#define cCmdMaskAndCode_XtdNotPossible 1
#define cCmdMaskAndCode_OnlyStdOrXtdPossible 2

/*--------------------------*/

typedef struct
{
  mLX_ObjHead;
  unsigned char Channel;
} tLX_CanL2GetChannelInfoReq;
/*--------------------------*/

typedef struct
{
  mLX_ObjHead;
  unsigned char Channel;
  unsigned char ChipType;
  unsigned char DefConfReg;
  unsigned long DefChipClock;
} tLX_CanL2GetChannelInfoConf;
/*--------------------------*/

typedef struct
{
  mLX_ObjHead;
  unsigned char Channel;
  unsigned char SpeedCode;
  unsigned char BTR0;
  unsigned char BTR1;
  unsigned char ConfReg;
  unsigned char SpecialModes;
  unsigned char notused;
  unsigned short ReqTimeStampRes;
} tLX_CanL2SetupReq;
/*--------------------------*/

typedef struct
{
  mLX_ObjHead;
  unsigned char Channel;
  unsigned short AssTimeStampRes;
} tLX_CanL2SetupConf;
/*--------------------------*/

typedef struct
{
  mLX_ObjHead;
  unsigned char Channel;
  twCanStdId StdMask;
  twCanStdId StdCode;
  tdwCanXtdId XtdMask;
  tdwCanXtdId XtdCode;
} tLX_CanL2SetMaskAndCodeReq;
/*--------------------------*/

typedef struct
{
  mLX_ObjHead;
  unsigned char Channel;
  unsigned char ErrorCode;
} tLX_CanL2SetMaskAndCodeConf;
/*--------------------------*/

typedef struct
{
  mLX_ObjHead;
  unsigned char Channel;
} tLX_CanL2BusOnReq;
/*--------------------------*/

typedef struct
{
  mLX_ObjHead;
  unsigned char Channel;
} tLX_CanL2BusOffReq;
/*--------------------------*/


typedef struct
{
  mLX_ObjHead;
  unsigned char Channel;
  unsigned char ChipStatus;
  unsigned char ChipError;
} tLX_CanL2ChipStatusChangedInd;
/*--------------------------*/

typedef struct
{
  mLX_ObjHead;
  unsigned char Channel;
  unsigned char ChipStatus;
  unsigned char ChipError;
  unsigned long TimeStamp;
} tLX_CanL2ChipStatusChangedTimeStampInd;
/*--------------------------*/

typedef struct
{
  mLX_ObjHead;
  unsigned char Channel;
} tLX_CanL2ReadErrorCounterReq;
/*--------------------------*/

typedef struct
{
  mLX_ObjHead;
  unsigned char Channel;
  unsigned short BusOffCounter;
  unsigned short WarnCounter;
} tLX_CanL2ReadErrorCounterConf;
/*--------------------------*/

typedef struct
{
  mLX_ObjHead;
  unsigned char Channel;
} tLX_CanL2ChipReceiverOverFlowInd;
/*--------------------------*/

typedef struct
{
  mLX_ObjHead;
  unsigned char Channel;
  unsigned char notused;
  unsigned char FrameInfo;
  tdwIdField IdField;
  unsigned char Data[8];
} tLX_CanL2TxObjectReq;
/*--------------------------*/

typedef struct
{
  mLX_ObjHead;
  unsigned char Channel;
  unsigned char notused;
  unsigned char FrameInfo;
  tdwIdField IdField;
  unsigned char Data[8];
} tLX_CanL2TxObjectConf;
/*--------------------------*/

typedef struct
{
  mLX_ObjHead;
  unsigned char Channel;
  unsigned char notused;
  unsigned char FrameInfo;
  tdwIdField IdField;
  unsigned char Data[8];
  unsigned long TimeStamp;
} tLX_CanL2TxObjectTimeStampConf;
/*--------------------------*/

typedef struct
{
  mLX_ObjHead;
  unsigned char Channel;
  unsigned char notused;
  unsigned char FrameInfo;
  tdwIdField IdField;
  unsigned char Data[8];
} tLX_CanL2RxObjectInd;
/*--------------------------*/

typedef struct
{
  mLX_ObjHead;
  unsigned char Channel;
  unsigned char notused;
  unsigned char FrameInfo;
  tdwIdField IdField;
  unsigned char Data[8];
  unsigned long TimeStamp;
} tLX_CanL2RxObjectTimeStampInd;
/*--------------------------*/

typedef struct
{
  mLX_ObjHead;
  unsigned char Channel;
} tLX_CanL2GetTxFifoSizeReq;
typedef struct
{
  mLX_ObjHead;
  unsigned char Channel;
  unsigned short TxFifoSize;
} tLX_CanL2GetTxFifoSizeConf;
/*--------------------------*/

typedef struct
{
  mLX_ObjHead;
  unsigned char Channel;
  unsigned char PhyDrv;
} tLX_CanL2SelectPhyDrvReq;
typedef struct
{
  mLX_ObjHead;
  unsigned char Channel;
  unsigned char PhyDrv;
} tLX_CanL2SelectPhyDrvConf;
#define cLX_PhyDrvSWCLowSpeed 0
#define cLX_PhyDrvSWCHighSpeed 1
#define cLX_PhyDrvMidSpeed 2
#define cLX_PhyDrvHighSpeed 3
#define cLX_PhyDrvRead 0xff
/*--------------------------*/

typedef struct
{
  mLX_ObjHead;
  unsigned char Channel;
  unsigned char PhyDrvSWCVoltage;
} tLX_CanL2SetPhyDrvSwcHighVoltReq;
typedef struct
{
  mLX_ObjHead;
  unsigned char Channel;
  unsigned char PhyDrvSWCVoltage;
} tLX_CanL2SetPhyDrvSwcHighVoltConf;
#define cLX_PhyDrvSWCVoltageNormal 0
#define cLX_PhyDrvSWCVoltageHigh 1
#define cLX_PhyDrvSWCVoltageRead 0xff
/*--------------------------*/

#endif
