/*
--------------------------------------------------------------------------

  LXDEMO2.CPP
    Example to show the LevelX API DLL usage with the
    CAN ISO layer 2 group

    working with a thread to react on events and read objects

ih980724 - first version
ih20000420 - try to read the DLL path from the registry
ih20011218- DriverClose moved onto correct place
ih20020107- TxTest added-> check a special std ID Data0 incrementation
ih20020611- ComTest added-> tx+check a special std ID Data0 incrementation
  TxConfirmation must be on, Tx:ID=0, Data0++  --> Rx: check it
ih20021015- Try to read "ChannelsI9141Count" from registry and take "CL21"
  or "C1K1" as short firmware name part for DriverOpen
ih20030516- LXOBJDEF.H removed, some beauty modifications,
  usage of CanL2GetTxFifoSizeReq and fast transmission with
  TxFifoSize-supervision added
--------------------------------------------------------------------------
*/

#include <dos.h>
#include <stdlib.h>
#include <stdio.h>
#include <conio.h>
#include <time.h>
#include <ctype.h>
#include <process.h>

#include "levelx.h"


//************************************
//************************************
#define mRxTest 0
#define mAutomaticTx 0

//************************************
#ifndef mRxTest
  #error "mRxTest must be defined"
#endif

#if (mRxTest)
  #define Test_IDstd 0ul
  static unsigned char LastRxedData0;
#endif

#ifndef mAutomaticTx
  #error "mAutomaticTx must be defined"
#endif
//************************************
//************************************


static unsigned char coorLxRdThreadIsActive = 0;
static HANDLE hLxRdEvent = NULL;
static HINSTANCE hLib = NULL;
static tLXSETEVENT LX_SetEvent = NULL;
static tLXWROBJECT LX_WrObject = NULL;
static tLXRDOBJECT LX_RdObject = NULL;
static char gszErrorFromThread[81] = "";

#define mChannelsMax 2
static unsigned char volatile boBusOn0 = 0;
#define mTxFwFifoFreeObjs_Init 9
static unsigned short volatile awTxFwFifoFreeObjsMax[mChannelsMax];
static unsigned short volatile awTxFwFifoFreeObjs[mChannelsMax];

static unsigned char volatile bTestErrorCounter = 0;

static unsigned long dwObjCounter = 0;
static unsigned long dwObjCountStartTime = 0;
static unsigned long dwObjPerSec = 0;

#define MAX_ObjSize ((unsigned short)256)
typedef unsigned char tObj[MAX_ObjSize];
// object buffer: obj from thread to decode + display
#define MAX_McObj ((unsigned short)4000)
tObj aMcObj[MAX_McObj];
static volatile unsigned short iWrMcObj = 0;
static volatile unsigned short iRdMcObj = 0;


/*------------------------------------------------------------------------*/
#define IdFieldFromXtdId(id) \
  ((((unsigned long)id<<27)&0xff000000ul)\
  |(((unsigned long)id<<11)&0x00ff0000ul)\
  |(((unsigned long)id>> 5)&0x0000ff00ul)\
  |(((unsigned long)id>>21)&0x000000fful))
#define IdFieldFromStdId(id) \
  ((((unsigned long)id<<13)&0x0000ff00ul)\
  |(((unsigned long)id>> 3)&0x000000fful))
/*------------------------------------------------------------------------*/


/*------------------------------------------------------------------------*/
static unsigned long IdFromIdField (
  unsigned char Xtd,
  unsigned long IdField)
/*------------------------------------------------------------------------*/
{
    typedef union
    {
      unsigned long dw;
      unsigned char ab[4];
    } tudw;
    tudw udw;
    tudw udw_in;

  udw_in.dw = IdField;
  if (Xtd)
  {
    udw.ab[0] = udw_in.ab[3];
    udw.ab[1] = udw_in.ab[2];
    udw.ab[2] = udw_in.ab[1];
    udw.ab[3] = udw_in.ab[0];
    udw.dw >>= 3;
  }
  else //Std
  {
    udw.dw = 0;
    udw.ab[0] = udw_in.ab[1];
    udw.ab[1] = udw_in.ab[0];
    udw.dw >>= 5;
  }
  return udw.dw;
}


/*------------------------------------------------------------------------*/
static void DisplayLxObjHex (tObj *pObj)
/*------------------------------------------------------------------------*/
{
  unsigned char len;
  unsigned short i;

  len = ((unsigned char *)pObj)[0];
  printf ("LxObj:");
  for (i = 0; i < len; i++)
  {
    printf (" %02x", ((unsigned char *)pObj)[i]);
  }
  printf ("\n");
}


/*------------------------------------------------------------------------*/
static void DispDriverOpenConf (tObj *pObj)
/*------------------------------------------------------------------------*/
{
  tLX_DriverOpenConf *pRdObj;

  pRdObj = (tLX_DriverOpenConf *)pObj;
  if (pRdObj->ErrorString[0])
    printf ("Open ErrorString: %s\n", pRdObj->ErrorString);
}

/*------------------------------------------------------------------------*/
static void DispDriverGetInterfaceInfoConf (tObj *pObj)
/*------------------------------------------------------------------------*/
{
  tLX_DriverGetInterfaceInfoConf *pRdObj;

  pRdObj = (tLX_DriverGetInterfaceInfoConf *)pObj;
  printf ("InterfaceInfo\n");
  printf ("  InterfaceType: %u\n", pRdObj->InterfaceType);
  printf ("  Address: 0x%08lx\n", pRdObj->Address);
  printf ("  PortNumber: %u\n", pRdObj->PortNumber);
  printf ("  BoardCode: >%s<\n", pRdObj->BoardCode);
  printf ("  ShortBoardName: >%s<\n", pRdObj->ShortBoardName);
  printf ("  LongBoardName: >%s<\n", pRdObj->LongBoardName);
}

/*------------------------------------------------------------------------*/
static void DispGlobGetDeviceInfoConf(tObj *pObj)
/*------------------------------------------------------------------------*/
{
  tLX_GlobGetDeviceInfoConf *pRdObj;

  pRdObj = (tLX_GlobGetDeviceInfoConf *)pObj;
  printf ("DeviceInfo\n");
  printf ("  FirmwareCode: %s\n", pRdObj->FirmwareCode);
  printf ("  ShortFirmwareName: %s\n", pRdObj->ShortFirmwareName);
  printf ("  VersionCode: %s\n", pRdObj->VersionCode);
  printf ("  LongFirmwareName: %s\n", pRdObj->LongFirmwareName);
}

/*------------------------------------------------------------------------*/
static void DispGlobGeneralErrorInd(tObj *pObj)
/*------------------------------------------------------------------------*/
{
  tLX_GlobGeneralErrorInd *pRdObj;

  pRdObj = (tLX_GlobGeneralErrorInd *)pObj;
  printf ("GeneralError: %s\n", pRdObj->ErrorString);
}

/*------------------------------------------------------------------------*/
static void DispCanL2GetChannelInfoConf (tObj *pObj)
/*------------------------------------------------------------------------*/
{
  tLX_CanL2GetChannelInfoConf *pRdObj;

  pRdObj = (tLX_CanL2GetChannelInfoConf *)pObj;
  printf ("ChannelInfo: Channel:%1u ChipType:%1u DefConfReg:0x%02x "
    "DefChipClock:%10lu kHz\n",
    pRdObj->Channel, pRdObj->ChipType, pRdObj->DefConfReg,
    pRdObj->DefChipClock);
}

/*------------------------------------------------------------------------*/
static void DispCanL2SetupConf (tObj *pObj)
/*------------------------------------------------------------------------*/
{
  tLX_CanL2SetupConf *pRdObj;

  pRdObj = (tLX_CanL2SetupConf *)pObj;
  printf ("CanL2Setup: Channel:%1u AssTimeStampRes:%5u us\n",
    pRdObj->Channel, pRdObj->AssTimeStampRes);
}

/*------------------------------------------------------------------------*/
static void DispCanL2SetMaskAndCodeConf (tObj *pObj)
/*------------------------------------------------------------------------*/
{
  tLX_CanL2SetMaskAndCodeConf *pRdObj;

  pRdObj = (tLX_CanL2SetMaskAndCodeConf *)pObj;
  printf ("CanL2SetMaskAndCode: Channel:%1u ", pRdObj->Channel);
  switch (pRdObj->ErrorCode)
  {
    case 0:
      printf("ok\n");
    break;
    case 1:
      printf("Error: extended frames are not possible\n");
    break;
    case 2:
      printf("Error: only standard or extended frames are possible\n");
    break;
    default:
      printf("unknown ErrorCode: 0x%02x\n", pRdObj->ErrorCode);
    break;
  }
}

/*------------------------------------------------------------------------*/
static void DispCanL2ChipStatusChangedInd (tObj *pObj)
/*------------------------------------------------------------------------*/
{
  tLX_CanL2ChipStatusChangedTimeStampInd *pRdObj;

  pRdObj = (tLX_CanL2ChipStatusChangedTimeStampInd *)pObj;
  printf ("CanL2ChipStatusChanged: Channel:%1u", pRdObj->Channel);
  if (pRdObj->ChipStatus & (1<<7))
    printf ("BusOff");
  else
    printf ("------");
  if (pRdObj->ChipStatus & (1<<6))
    printf (" WarnState");
  else
    printf (" ---------");
  printf(" ChipError:0x%02x", pRdObj->ChipError);
  if (pRdObj->ObjectID == LX_OID_CanL2ChipStatusChangedTimeStampInd)
    printf(" Time:%10lu", pRdObj->TimeStamp);
  printf("\n");
}

/*------------------------------------------------------------------------*/
static void DispCanL2ChipReceiverOverFlowInd (tObj *pObj)
/*------------------------------------------------------------------------*/
{
  tLX_CanL2ChipReceiverOverFlowInd *pRdObj;

  pRdObj = (tLX_CanL2ChipReceiverOverFlowInd *)pObj;
  printf ("CanL2ChipReceiverOverFlow: Channel:%1u\n",
    pRdObj->Channel);
}

/*------------------------------------------------------------------------*/
static void DispCanL2RxTxObject (tObj *pObj)
/*------------------------------------------------------------------------*/
{
  tLX_CanL2RxObjectTimeStampInd *pRdObj;
  unsigned char b;
  unsigned short w;
  unsigned long dw;
  unsigned char bRxTestErr;
  unsigned char bCh;

  bRxTestErr = 0;
  pRdObj = (tLX_CanL2RxObjectTimeStampInd *)pObj;
  bCh = pRdObj->Channel;
  b = pRdObj->FrameInfo;
  dw = IdFromIdField ((b & (unsigned char)(1<<7)), pRdObj->IdField);
  if ((pRdObj->ObjectID == LX_OID_CanL2TxObjectConf)
    || (pRdObj->ObjectID == LX_OID_CanL2TxObjectTimeStampConf))
  {
    if (awTxFwFifoFreeObjs[bCh] < awTxFwFifoFreeObjsMax[bCh])
      awTxFwFifoFreeObjs[bCh]++;

    //---------------------------------------------------------
    //--RxTest-------------------------------------------------
    #if (mRxTest)
      if (bCh == 0)
      {
        //-- check right frame received
        if (
          !(b & (1<<7)) //--not Xtd ID
          && (dw == Test_IDstd)
          && !(b & (1<<6)) //--not RTR
          && ((b & 0x0f) > 0) //--Len > 0
        )
        {
          unsigned char Data0;
          Data0 = pRdObj->Data[0];
          if ((unsigned char)((LastRxedData0 + (unsigned char)1)
            & (unsigned char)0xff) != Data0)
          {
            printf ("\n####++++----VVVVV Error: Data[0] incrementation, "
              "old:%02x new:%02x VVVVV\n", LastRxedData0, Data0);
            bRxTestErr = 1;
            bTestErrorCounter++;
          }
          else
          {
            dwObjCounter++;
            if (dwObjCountStartTime == 0) //init
            {
              dwObjCountStartTime = GetTickCount ();
              dwObjCounter = 0;
            }
            {
              unsigned long dwDeltaTime;
              dwDeltaTime = GetTickCount () - dwObjCountStartTime;
              if (dwDeltaTime == 0)
                dwObjPerSec = (unsigned long)-1;
              else
                dwObjPerSec = dwObjCounter * 1000 / dwDeltaTime;
            }
          }
          LastRxedData0 = Data0;
        }
      }
    #endif
    //--RxTest-------------------------------------------------
    //---------------------------------------------------------

    if ((dwObjCounter % 10) == 0)
      printf ("TX");
  }
  else
    printf ("RX");
  if ((dwObjCounter % 10) == 0)
  {
    printf("%1u ID:", bCh);
    if (b & (1<<7))
      printf("X");
    else
      printf("S");
    printf ("-Id%10lu=0x%08lx", dw, dw);
    printf (" Len:%1u", b & 0x0f);
    if (b & (1<<6))  //Rtr
    {
      printf(" RTR");
    }
    else // Data
    {
      b &= 0x0f;
      for (w = 0; w < b; w++)
        printf (" %02x", pRdObj->Data[w]);
    }
    if ((pRdObj->ObjectID == LX_OID_CanL2RxObjectTimeStampInd)
      || (pRdObj->ObjectID == LX_OID_CanL2TxObjectTimeStampConf))
      printf(" Time:%10lu", pRdObj->TimeStamp);
    #if (mRxTest)
      if (bRxTestErr)
        printf ("\n");
      else
        printf (" - %10lu ops\r", dwObjPerSec);
    #else
      printf ("\n");
    #endif
  }
}


/*------------------------------------------------------------------------*/
static void DispCanL2GetTxFifoSizeConf (tObj *pObj)
/*------------------------------------------------------------------------*/
{
  tLX_CanL2GetTxFifoSizeConf *pRdObj;
  unsigned char bCh;

  pRdObj = (tLX_CanL2GetTxFifoSizeConf *)pObj;
  bCh = pRdObj->Channel;
  if (bCh >= mChannelsMax)
    DisplayLxObjHex (pObj);
  else
  {
    unsigned short wTxFwFifoUsedObjs;
    wTxFwFifoUsedObjs = awTxFwFifoFreeObjsMax[bCh] - awTxFwFifoFreeObjs[bCh];
    awTxFwFifoFreeObjsMax[bCh] = pRdObj->TxFifoSize;
    awTxFwFifoFreeObjs[bCh] = awTxFwFifoFreeObjsMax[bCh] - wTxFwFifoUsedObjs;

    printf ("CanL2GetTxFifoSize: Channel:%1u - got TxFifoSize:%1u\n",
      bCh, awTxFwFifoFreeObjsMax[bCh]);
  }
}


/*------------------------------------------------------------------------*/
static void DisplayLxObj (tObj *pObj)
/*------------------------------------------------------------------------*/
{
  switch (((unsigned char *)pObj)[1])  // GroupID
  {
    case LX_GID_Driver:
      switch (((unsigned char *)pObj)[2]) // ObjectID
      {
        case LX_OID_DriverOpenConf:
          DispDriverOpenConf (pObj);
        break;
        case LX_OID_DriverGetInterfaceInfoConf:
          DispDriverGetInterfaceInfoConf (pObj);
        break;
        default:
          DisplayLxObjHex(pObj);
        break;
      }
    break;
    case LX_GID_DirectAccess:
      DisplayLxObjHex(pObj);
    break;
    case LX_GID_Glob:
      switch (((unsigned char *)pObj)[2]) // ObjectID
      {
        case LX_OID_GlobGetDeviceInfoConf:
          DispGlobGetDeviceInfoConf(pObj);
        break;
        case LX_OID_GlobGeneralErrorInd:
          DispGlobGeneralErrorInd(pObj);
        break;
        default:
          DisplayLxObjHex(pObj);
        break;
      }
    break;
    case LX_GID_CanL2:
      switch (((unsigned char *)pObj)[2]) // ObjectID
      {
        case LX_OID_CanL2GetChannelInfoConf:
          DispCanL2GetChannelInfoConf(pObj);
        break;
        case LX_OID_CanL2SetupConf:
          DispCanL2SetupConf(pObj);
        break;
        case LX_OID_CanL2SetMaskAndCodeConf:
          DispCanL2SetMaskAndCodeConf(pObj);
        break;
        case LX_OID_CanL2ChipStatusChangedInd:
        case LX_OID_CanL2ChipStatusChangedTimeStampInd:
          DispCanL2ChipStatusChangedInd(pObj);
        break;
        case LX_OID_CanL2ChipReceiverOverFlowInd:
          DispCanL2ChipReceiverOverFlowInd(pObj);
        break;
        case LX_OID_CanL2RxObjectInd:
        case LX_OID_CanL2RxObjectTimeStampInd:
        case LX_OID_CanL2TxObjectConf:
        case LX_OID_CanL2TxObjectTimeStampConf:
          DispCanL2RxTxObject(pObj);
        break;
        case LX_OID_CanL2GetTxFifoSizeConf:
          DispCanL2GetTxFifoSizeConf(pObj);
        break;
        default:
          DisplayLxObjHex(pObj);
        break;
      }
    break;
    default:
      DisplayLxObjHex(pObj);
    break;
  }
}


/*------------------------------------------------------------------------*/
static unsigned char DisplayNextLxObj (void)
/*------------------------------------------------------------------------*/
{
  tObj *pObj;

  if (iRdMcObj != iWrMcObj)
  {
    pObj = (tObj *)(aMcObj[iRdMcObj]);
    //----------------------------
    DisplayLxObj (pObj);
    //----------------------------
    iRdMcObj++;
    if (iRdMcObj >= MAX_McObj)
      iRdMcObj = 0;
    return 1;
  }
  return 0;
}


/*------------------------------------------------------------------------*/
static void BufferLxObj (tObj *pObj)
/*------------------------------------------------------------------------*/
{
  unsigned short iWrMcObjNew;

  iWrMcObjNew = iWrMcObj + 1;
  if (iWrMcObjNew >= MAX_McObj)
    iWrMcObjNew = 0;
  if (iRdMcObj != iWrMcObjNew)
  {
    memcpy (aMcObj[iWrMcObj], pObj, (unsigned char)(pObj[0]));
    iWrMcObj = iWrMcObjNew;
  }
  else
  {
    printf("\n");
    printf("application buffer overflow !!!\n");
  }
}


/*------------------------------------------------------------------------*/
static unsigned char TestBufferFullLxObj (void)
/*------------------------------------------------------------------------*/
{
  unsigned short iWrMcObjNew;

  iWrMcObjNew = iWrMcObj + 1;
  if (iWrMcObjNew >= MAX_McObj)
    iWrMcObjNew = 0;
  if (iRdMcObj != iWrMcObjNew)
    return 0; //ok
  else
    return 1; //full
}


/*------------------------------------------------------------------------*/
static unsigned char ShowError (char *szError)
/*------------------------------------------------------------------------*/
{
  if (szError[0])
  {
    printf ("ERROR: %s\n", szError);
    return 1;
  }
  return 0;
}


/*------------------------------------------------------------------------*/
static void GetLxErrString (char *szError, tLxErr Err)
/*------------------------------------------------------------------------*/
{
  char *lx_err[] = {
    "",
    "LXERR_RD_NO_OBJ_AVAILABLE",
    "LXERR_WR_BUFFER_FULL",
    "LXERR_SET_EVENT",
    "LXERR_CHANNEL_OPEN",
    "LXERR_BOARDNUMBER",
    "LXERR_ILLEGALOBJECT"
  };

  strcpy(szError, lx_err[Err]);
}


/*------------------------------------------------------------------------*/
static unsigned char WrObject(
  void *Obj,
  char *szError)
/*------------------------------------------------------------------------*/
{
  tLxErr Err;

  szError[0] = 0;
  Err = LX_WrObject (Obj);
  if (Err)
  {
    GetLxErrString (szError, Err);
    return 0;
  }
  return 1;
}


/*------------------------------------------------------------------------*/
static unsigned char RdObject(
  void *Obj,
  char *szError)
/*------------------------------------------------------------------------*/
{
  tLxErr Err;

  szError[0] = 0;
  Err = LX_RdObject (Obj);
  if (Err == LXERR_RD_NO_OBJ_AVAILABLE)
    return 0;
  if (Err)
  {
    GetLxErrString (szError, Err);
    return 0;
  }
  return 1;
}


/*------------------------------------------------------------------------*/
static void DriverOpen (
  unsigned char byDriverOpenBoardNumber,
  char *szShortFirmwareName,
  char *szSpecialFirmware,
  char *szError)
/*------------------------------------------------------------------------*/
{
  unsigned char RdObj[MAX_ObjSize];
  tLX_DriverOpenReq WrObj;

  WrObj.ObjLen = sizeof(tLX_DriverOpenReq);
  WrObj.GroupID = LX_GID_Driver;
  WrObj.ObjectID = LX_OID_DriverOpenReq;
  WrObj.BoardNumber = byDriverOpenBoardNumber;
  strcpy (WrObj.ShortFirmwareName, szShortFirmwareName);
  strcpy (WrObj.SpecialFirmware, szSpecialFirmware);

  // Request
  if (WrObject(&WrObj, szError))
  {
    // Response
    while (1)
    {
      Sleep(10);
      if (RdObject(RdObj, szError))
      {
        if ((RdObj[1] == LX_GID_Driver) && (RdObj[2] == LX_OID_DriverOpenConf))
        {
          strcpy (szError, (*(tLX_DriverOpenConf *)RdObj).ErrorString);
          break;
        }
        else
          printf ("not expected object LEN:0x%02x GID:0x%02x OID:0x%02x\n",
                  RdObj[0], RdObj[1], RdObj[2]);
        break;
      }
      else if (szError[0])
        break;
    }
  }
}


/*------------------------------------------------------------------------*/
static void DriverClose (
  char *szError)
/*------------------------------------------------------------------------*/
{
  tLX_DriverCloseReq WrObj;

  WrObj.ObjLen = sizeof(tLX_DriverCloseReq);
  WrObj.GroupID = LX_GID_Driver;
  WrObj.ObjectID = LX_OID_DriverCloseReq;
  WrObject(&WrObj, szError);
}


/*------------------------------------------------------------------------*/
static void DriverGetInterfaceInfo (
  char *szError)
/*------------------------------------------------------------------------*/
{
  unsigned char RdObj[MAX_ObjSize];
  tLX_DriverGetInterfaceInfoReq WrObj;

  WrObj.ObjLen = sizeof(tLX_DriverGetInterfaceInfoReq);
  WrObj.GroupID = LX_GID_Driver;
  WrObj.ObjectID = LX_OID_DriverGetInterfaceInfoReq;

  // Request
  if (WrObject(&WrObj, szError))
  {
    // Response
    while (1)
    {
      Sleep(10);
      if (RdObject(RdObj, szError))
      {
        if ((RdObj[1] == LX_GID_Driver) &&
            (RdObj[2] == LX_OID_DriverGetInterfaceInfoConf))
        {
          DispDriverGetInterfaceInfoConf ((tObj *)&RdObj[0]);
          break;
        }
        else
          printf ("not expected object LEN:0x%02x GID:0x%02x OID:0x%02x\n",
                  RdObj[0], RdObj[1], RdObj[2]);
        break;
      }
      else if (szError[0])
        break;
    }
  }
}


/*------------------------------------------------------------------------*/
static void GlobGetDeviceInfoReq(
  char *szError)
/*------------------------------------------------------------------------*/
{
  tLX_GlobGetDeviceInfoReq WrObj;
  tLxErr Err;

  WrObj.ObjLen = sizeof(tLX_GlobGetDeviceInfoReq);
  WrObj.GroupID = LX_GID_Glob;
  WrObj.ObjectID = LX_OID_GlobGetDeviceInfoReq;
  szError[0] = 0;
  Err = LX_WrObject (&WrObj);
  if (Err)
  {
    GetLxErrString (szError, Err);
  }
}


/*------------------------------------------------------------------------*/
static void PollAndDisplayLxObject(unsigned short Time_100ms, char *szError)
/*------------------------------------------------------------------------*/
{
  tObj RdObj;

  while (!RdObject(RdObj, szError))
  {
    if (!Time_100ms)
    {
      printf ("PollAndDisplayLxObject - timeout\n");
      return;
    }
    Time_100ms--;
    Sleep (100); //ms
  }
  DisplayLxObj ((tObj *)&RdObj[0]);
}


/*------------------------------------------------------------------------*/
void LxRdThread (void *ArgbyBoard)
/*------------------------------------------------------------------------*/
{
  unsigned char RdObj[MAX_ObjSize];
  HANDLE hEvent;

  hEvent = CreateEvent (NULL, FALSE, FALSE, NULL);
  hLxRdEvent = hEvent;
  coorLxRdThreadIsActive = 1;
  LX_SetEvent (hEvent);
  while(1)
  {
    WaitForSingleObject(hEvent, 2000);
    if (coorLxRdThreadIsActive != 1)
      break;
    LX_SetEvent (hEvent);
    /*-----------------------------------------------------------*/
    // read transfer objects
    while (1)
    {
      // avoid crash on terminating application
      if (coorLxRdThreadIsActive != 1)
        break;
/*
      // test object buffer
      if (TestBufferFullLxObj())
        break; // buffer in application full
*/
      if (TestBufferFullLxObj())
        Sleep (500);

      // read object
      if (RdObject(RdObj, gszErrorFromThread))
      {
        BufferLxObj((tObj *)&RdObj[0]);
      }
      else
        break;
    }
    /*-----------------------------------------------------------*/
  }
  CloseHandle(hEvent);
  coorLxRdThreadIsActive = 0;
  _endthread();
}


/*------------------------------------------------------------------------*/
static void LxRdCreateThread (
  char *szError)
/*------------------------------------------------------------------------*/
{
  unsigned long thread_id;
  unsigned short timeout;

  szError[0] = 0;
  if (coorLxRdThreadIsActive == 0)
  {
    if ((thread_id = _beginthread (LxRdThread, 4096, (void *)0))
         == (unsigned long)-1)
    {
      sprintf (szError, "Unable to create LxRdThread");
    }
  }
  timeout = 100;
  while ((timeout--))
  {
    Sleep(10);
    if (coorLxRdThreadIsActive != 0)
      break;
  }
}


/*------------------------------------------------------------------------*/
static void LxRdEndThread (void)
/*------------------------------------------------------------------------*/
{
  unsigned short timeout;

  coorLxRdThreadIsActive = 2;
  if (hLxRdEvent)
  {
    PulseEvent (hLxRdEvent);
  }
  timeout = 100;
  while ((timeout--))
  {
    Sleep(10);
    if (coorLxRdThreadIsActive == 0)
      break;
  }
}


/*------------------------------------------------------------------------*/
void DispUsage (void)
/*------------------------------------------------------------------------*/
{
  printf (" usage: LXDEMOx DLL-name board/portnumber number_of_channels\n");
  printf ("         SpeedCode BTR0 BTR1 ConfReg SpecialModes        (for channel 0)\n");
  printf ("         [SpeedCode BTR0 BTR1 ConfReg SpecialModes]      (opt. for channel 1)\n");
  printf ("  SpeedCode: 1/0=On/Off for ErrorIndication+AutoBusOn+TxConfirmation+TimeStamps");
  printf (" demo: lxdemo1 lxn4pi2j 0 2 255 c3 49 fa 1001 255 c3 49 fa 1001\n");
}


/*------------------------------------------------------------------------*/
void main (int argc, char *argv[])
/*------------------------------------------------------------------------*/
{
  char szError[81];
  int key=0;
  tLX_CanL2TxObjectReq aTxObj[2];
  char szDllName[1024] = "";
  char *pszShortFirmwareName_FunctionPart;
  unsigned char byDriverOpenBoardNumber = 0;
  unsigned char byNumberOfChannels = 0;
  tLX_CanL2SetupReq aSetup[2];
  int i;

  while (1)
  {
    szError[0] = 0; //empty
    boBusOn0 = 0;
    {
      unsigned char bCh;
      for (bCh = 0; bCh < mChannelsMax; bCh++)
      {
        awTxFwFifoFreeObjsMax[bCh] = mTxFwFifoFreeObjs_Init;
        awTxFwFifoFreeObjs[bCh] = awTxFwFifoFreeObjsMax[bCh];
      }
    }
    bTestErrorCounter = 0;
    dwObjCounter = 0;
    dwObjCountStartTime = 0;

    // check the parameters
    if ((argc != 9) && (argc != 14))
    {
      printf ("\nLxDemo wrong number of parameter\n");
      DispUsage ();
      _getch();
      return;
    }
    if ((argc != 14) && (argv[3][0] == '2'))
    {
      printf ("\nLxDemo too few parameter for 2 channels\n");
      DispUsage ();
      _getch();
      return;
    }
    if (strlen(argv[8]) != 4)
    {
      printf ("\nLxDemo Channel0-SpeedCode must have 4 chars\n");
      DispUsage ();
      _getch();
      return;
    }
    if ((argc == 14) && (strlen(argv[13]) != 4))
    {
      printf ("\nLxDemo Channel1-SpeedCode must have 4 chars\n");
      DispUsage ();
      _getch();
      return;
    }
    pszShortFirmwareName_FunctionPart = "CL21";
#if 1
    // read parameters
    {
      // try to read DLL path from the Registry
      HKEY hReg = NULL;
      char szRegEntry[1024];

      sprintf (szRegEntry,
        "SOFTWARE\\I+ME ACTIA GmbH\\LevelX\\Interfaces\\%s", argv[1]);
      szDllName[0] = 0;
      if (RegOpenKeyEx (HKEY_LOCAL_MACHINE, szRegEntry, 0, KEY_READ,
          &hReg) == ERROR_SUCCESS)
      {
        {
          unsigned long dwCount = sizeof(szDllName);

          if (RegQueryValueEx (hReg, "Dll", 0, NULL, (unsigned char *)szDllName, &dwCount)
              != ERROR_SUCCESS)
            szDllName[0] = 0;
        }
        {
          char szChannelsI9141Count[32] = "";
          unsigned long dwCount = sizeof(szChannelsI9141Count);

          if (RegQueryValueEx (hReg, "ChannelsI9141Count", 0, NULL,
              (unsigned char *)szChannelsI9141Count, &dwCount) != ERROR_SUCCESS)
            szChannelsI9141Count[0] = 0;
          if (szChannelsI9141Count[0] == '1')
            pszShortFirmwareName_FunctionPart = "C1K1";
          else
            pszShortFirmwareName_FunctionPart = "CL21";
        }
        RegCloseKey (hReg);
      }
    }
    if (!szDllName[0]) //if not found in the Registry take it without path
#endif
      sprintf (szDllName, "%s.dll", argv[1]);
    byDriverOpenBoardNumber = argv[2][0] - '0';
    if (byDriverOpenBoardNumber > 9)
      byDriverOpenBoardNumber = 9;
    byNumberOfChannels = argv[3][0] - '0';
    if (byNumberOfChannels > 2)
      byNumberOfChannels = 2;
    if (byNumberOfChannels < 1)
      byNumberOfChannels = 1;

    aSetup[0].ObjLen = sizeof(tLX_CanL2SetupReq);
    aSetup[0].GroupID = LX_GID_CanL2;
    aSetup[0].ObjectID = LX_OID_CanL2SetupReq;
    aSetup[0].Channel = 0;
    sscanf (argv[4], "%u", &i);
    aSetup[0].SpeedCode = i;
    sscanf (argv[5], "%x", &i);
    aSetup[0].BTR0 = i;
    sscanf (argv[6], "%x", &i);
    aSetup[0].BTR1 = i;
    sscanf (argv[7], "%x", &i);
    aSetup[0].ConfReg = i;
    {
      unsigned char b = 0;
      b = 0x0f;
      if (argv[8][0] == '0')
        b &= ~(1<<0);
      if (argv[8][1] == '0')
        b &= ~(1<<1);
      if (argv[8][2] == '0')
        b &= ~(1<<2);
      if (argv[8][3] == '0')
        b &= ~(1<<3);
      aSetup[0].SpecialModes = b;
    }
    aSetup[0].ReqTimeStampRes = 100; //us

    if (argc == 14)
    {
      aSetup[1].ObjLen = sizeof(tLX_CanL2SetupReq);
      aSetup[1].GroupID = LX_GID_CanL2;
      aSetup[1].ObjectID = LX_OID_CanL2SetupReq;
      aSetup[1].Channel = 1;
      sscanf (argv[9], "%u", &i);
      aSetup[1].SpeedCode = i;
      sscanf (argv[10], "%x", &i);
      aSetup[1].BTR0 = i;
      sscanf (argv[11], "%x", &i);
      aSetup[1].BTR1 = i;
      sscanf (argv[12], "%x", &i);
      aSetup[1].ConfReg = i;
      {
        unsigned char b = 0;
        b = 0x0f;
        if (argv[13][0] == '0')
          b &= ~(1<<0);
        if (argv[13][1] == '0')
          b &= ~(1<<1);
        if (argv[13][2] == '0')
          b &= ~(1<<2);
        if (argv[13][3] == '0')
          b &= ~(1<<3);
        aSetup[1].SpecialModes = b;
      }
      aSetup[1].ReqTimeStampRes = 100; //us
    }

    // display entry informations
    printf ("\n");
    printf ("+++++ LxDemo2 +++++ "__DATE__"\n");
    printf ("- DLL: %s ", szDllName);
    printf ("- Board/Port: %u\n", byDriverOpenBoardNumber);

    // init CAN transmit objects
    aTxObj[0].ObjLen = sizeof(tLX_CanL2TxObjectReq);
    aTxObj[0].GroupID = LX_GID_CanL2;
    aTxObj[0].ObjectID = LX_OID_CanL2TxObjectReq;
    aTxObj[0].Channel = 0;
    aTxObj[0].FrameInfo = 2;
    aTxObj[0].IdField = IdFieldFromStdId (0);
    aTxObj[0].Data[0] = 0;
    aTxObj[0].Data[1] = 0x11;
    aTxObj[0].Data[2] = 0x12;

    aTxObj[1].ObjLen = sizeof(tLX_CanL2TxObjectReq);
    aTxObj[1].GroupID = LX_GID_CanL2;
    aTxObj[1].ObjectID = LX_OID_CanL2TxObjectReq;
    aTxObj[1].Channel = 1;
    aTxObj[1].FrameInfo = 3;
    aTxObj[1].IdField = IdFieldFromStdId (1);
    aTxObj[1].Data[0] = 0x80;
    aTxObj[1].Data[1] = 0x21;
    aTxObj[1].Data[2] = 0x22;
    aTxObj[1].Data[3] = 0x23;

    ///////////////////////////////////////////////////////////
    printf ("LoadLibrary\n");
    if ((hLib = LoadLibrary (szDllName)) == NULL)
      sprintf(szError, "LoadLibrary %lu", GetLastError());
    else if ((LX_SetEvent = (tLXSETEVENT)GetProcAddress(hLib, "LXSETEVENT")) == NULL)
      sprintf(szError, "GetProcAddress LXSETEVENT %lu", GetLastError());
    else if ((LX_WrObject = (tLXWROBJECT)GetProcAddress(hLib, "LXWROBJECT")) == NULL)
      sprintf(szError, "GetProcAddress LXWROBJECT %lu", GetLastError());
    else if ((LX_RdObject = (tLXRDOBJECT)GetProcAddress(hLib, "LXRDOBJECT")) == NULL)
      sprintf(szError, "GetProcAddress LXRDOBJECT %lu", GetLastError());

    if (!szError[0])
    {
      ///////////////////////////////////////////////////////////
      printf ("DriverOpen (..Firmware download..)");
      DriverOpen (byDriverOpenBoardNumber, pszShortFirmwareName_FunctionPart,
        "", szError);
      printf (" ok\n");
      if (!szError[0])
      {
        printf ("DriverGetInterfaceInfo\n");
        DriverGetInterfaceInfo (szError);
      }
      if (!szError[0])
      {
        printf ("GlobGetDeviceInfoReq\n");
        GlobGetDeviceInfoReq(szError);
        ShowError (szError);
        PollAndDisplayLxObject(10, szError);
        ShowError (szError);
      }
      if (!szError[0])
      {
        ///////////////////////////////////////////////////////////
        printf ("LxRdCreateThread\n");
        LxRdCreateThread (szError);
        ShowError (szError);
      }
      if (!szError[0])
      {
        // --- Get TxFifoBufferSize channel 0
        {
          tLX_CanL2GetTxFifoSizeReq sCanL2GetTxFifoSizeReq;
          sCanL2GetTxFifoSizeReq.ObjLen =
            sizeof(tLX_CanL2GetTxFifoSizeReq);
          sCanL2GetTxFifoSizeReq.GroupID =
            LX_GID_CanL2;
          sCanL2GetTxFifoSizeReq.ObjectID =
            LX_OID_CanL2GetTxFifoSizeReq;
          sCanL2GetTxFifoSizeReq.Channel = 0;
          WrObject(&sCanL2GetTxFifoSizeReq, szError);
        }
        ShowError (szError);
        Sleep(100);
      }
      if ((!szError[0]) && (byNumberOfChannels > 1))
      {
        // --- Get TxFifoBufferSize channel 1
        {
          tLX_CanL2GetTxFifoSizeReq sCanL2GetTxFifoSizeReq;
          sCanL2GetTxFifoSizeReq.ObjLen =
            sizeof(tLX_CanL2GetTxFifoSizeReq);
          sCanL2GetTxFifoSizeReq.GroupID =
            LX_GID_CanL2;
          sCanL2GetTxFifoSizeReq.ObjectID =
            LX_OID_CanL2GetTxFifoSizeReq;
          sCanL2GetTxFifoSizeReq.Channel = 1;
          WrObject(&sCanL2GetTxFifoSizeReq, szError);
        }
        ShowError (szError);
        Sleep(100);
      }
      if (!szError[0])
      {
        // --- CanL2Setup channel 0
        WrObject((tObj *)&(aSetup[0]), szError);
        ShowError (szError);
        Sleep(100);
      }
      if ((!szError[0]) && (byNumberOfChannels > 1))
      {
        // --- CanL2Setup channel 1
        WrObject((tObj *)&(aSetup[1]), szError);
        ShowError (szError);
        Sleep(100);
      }

      //-- wait on confirmations and process them
      Sleep(200);
      while (DisplayNextLxObj());

      if (!szError[0])
      {
        printf ("----->press '4' to start...\n");


        //////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////
        while (1)
        {

          #if (mAutomaticTx)
            //-auto switch off on error -----
            if (bTestErrorCounter > 1)
            {
              unsigned char b = 0;
              tLX_CanL2BusOffReq sCanL2BusOffReq;

              printf("\n");
              printf("BusOff channel %1u\n", b);
              sCanL2BusOffReq.ObjLen =
                sizeof(tLX_CanL2BusOffReq);
              sCanL2BusOffReq.GroupID =
                LX_GID_CanL2;
              sCanL2BusOffReq.ObjectID =
                LX_OID_CanL2BusOffReq;
              sCanL2BusOffReq.Channel = b;
              WrObject(&sCanL2BusOffReq, szError);
              ShowError (szError);

              boBusOn0 = 0;
              bTestErrorCounter = 0;
            }
          #endif

          //-display-------------------
          while (DisplayNextLxObj());

          //-transmitt-----------------
          if (!boBusOn0)
            Sleep (50);
          else
          {
            #if (mAutomaticTx)
              while (awTxFwFifoFreeObjs[0] != 0)
              {
                awTxFwFifoFreeObjs[0]--;

                aTxObj[0].Data[0]++;
                WrObject((tObj *)&(aTxObj[0]), szError);
                ShowError (szError);
              }
              if (awTxFwFifoFreeObjs[0] == 0)
                Sleep (1);
            #endif
          }

          // user input handling -------------------------------------------
          if (kbhit())
          {
            key = tolower(getch());
            if (!key || key == 224 /*F11 or F12*/)
              key = getch() + 0x100;
          }
          else
    		    key = 0;
          if ((key == 'x') || (key == 27))
            break;
          switch (key)
          {
            case 0: // nothing to do
            break;
            case 'i':
              GlobGetDeviceInfoReq(szError);
              ShowError (szError);
            break;
            case '1':
            case '2':
            case '3':
            {
              unsigned char b = 0;
              tLX_CanL2GetChannelInfoReq sCanL2GetChannelInfoReq;

              b = key - '1';  // b = channel
              printf("GetChannelInfo channel %1u\n", b);
              sCanL2GetChannelInfoReq.ObjLen =
                sizeof(tLX_CanL2GetChannelInfoReq);
              sCanL2GetChannelInfoReq.GroupID =
                LX_GID_CanL2;
              sCanL2GetChannelInfoReq.ObjectID =
                LX_OID_CanL2GetChannelInfoReq;
              sCanL2GetChannelInfoReq.Channel = b;
              WrObject(&sCanL2GetChannelInfoReq, szError);
              ShowError (szError);
            }
            break;
            case '4':
              boBusOn0 = 1;
            case '5':
            {
              unsigned char b = 0;
              tLX_CanL2BusOnReq sCanL2BusOnReq;

              b = key - '4';  // b = channel
              printf("BusOn channel %1u\n", b);
              sCanL2BusOnReq.ObjLen =
                sizeof(tLX_CanL2BusOnReq);
              sCanL2BusOnReq.GroupID =
                LX_GID_CanL2;
              sCanL2BusOnReq.ObjectID =
                LX_OID_CanL2BusOnReq;
              sCanL2BusOnReq.Channel = b;
              WrObject(&sCanL2BusOnReq, szError);
              ShowError (szError);
            }
            break;
            case 'r':
              boBusOn0 = 0;
            case 't':
            {
              unsigned char b = 0;
              tLX_CanL2BusOffReq sCanL2BusOffReq;

              if (key == 'r')// b = channel
                b = 0;
              else
                b = 1;
              printf("BusOff channel %1u\n", b);
              sCanL2BusOffReq.ObjLen =
                sizeof(tLX_CanL2BusOffReq);
              sCanL2BusOffReq.GroupID =
                LX_GID_CanL2;
              sCanL2BusOffReq.ObjectID =
                LX_OID_CanL2BusOffReq;
              sCanL2BusOffReq.Channel = b;
              WrObject(&sCanL2BusOffReq, szError);
              ShowError (szError);
            }
            break;
            case '6':
            case '7':
            {
              unsigned char b = 0;

              b = key - '6';  // b = channel
              printf("Tx on channel %1u\n", b);
              aTxObj[b].Data[0]++;
              WrObject(&(aTxObj[b]), szError);
              ShowError (szError);
            }
            break;
            default:
              printf("Keys: x/ESC=exit              i    =DeviceInfo\n"
                     "      1/2/3=ChannelInfo0/1/2  4/5  =BusOn0/1\n"
                     "      r/t  =BusOff0/1         6/7  =Tx0/1\n"
                     "      8/9  =CanSetup0/1\n");
            break;
          }
          key = 0; // clear key command
        }
        //////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////
        //////////////////////////////////////////////////////////////


        printf ("LxRdEndThread\n");
        LxRdEndThread ();
      }
      printf ("DriverClose\n");
      {
        char szError2[81];
        szError2[0] = 0; //clear it
        DriverClose (szError2);
        ShowError (szError2);
      }
    }
    ShowError (szError);
    printf ("FreeLibrary\n");
    if (hLib)
      FreeLibrary(hLib);

    if (szError)
    {
      int key;
      printf ("LxDemo2-End, to try again press <r>, other key to exit.\n");
      key = _getch();
      if (key != 'r')
        break; // ----------->>>>>>>>>>>>>>>> exit
    }
    else
    {
      printf ("++++ LxConApp ++++ End reached, press enter\n");
      _getch();
      break;
    }
  } // while(1)
}
