#ifndef __LXGENDEF_H
#define __LXGENDEF_H

/*-----------------------------------------------------------------
   LevelX general definitions
   -- Driver
   -- DirectAccess
   -- Glob

990415 ih - GlobPorts added
991122 ih - ReadSerialNumber added
-----------------------------------------------------------------*/

/* !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
Note that the compiler must set on byte alignment!
For MS-C++ it is:
Project Settings.C/C++.Struct member aligment = 1 Byte
!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!! */

#include "lxcanl2.h"

/* --- object group identifier --- */
#define LX_GID_Driver                             ((unsigned char)0x00)
#define LX_GID_DirectAccess                       ((unsigned char)0x01)
#define LX_GID_Glob                               ((unsigned char)0x02)
#define LX_GID_CanL2                              ((unsigned char)0x03)


/* --- driver object identifier --- */
#define LX_OID_DriverOpenReq                      ((unsigned char)0x02)
#define LX_OID_DriverOpenConf                     ((unsigned char)0x03)
#define LX_OID_DriverCloseReq                     ((unsigned char)0x04)
#define LX_OID_DriverGetInterfaceInfoReq          ((unsigned char)0x06)
#define LX_OID_DriverGetInterfaceInfoConf         ((unsigned char)0x07)

/* --- direct access object identifier --- */
#define LX_OID_DirectAccessReadReq                ((unsigned char)0x02)
#define LX_OID_DirectAccessReadConf               ((unsigned char)0x03)
#define LX_OID_DirectAccessWriteReq               ((unsigned char)0x04)
#define LX_OID_DirectAccessWriteConf              ((unsigned char)0x05)

/* --- global object identifier --- */
#define LX_OID_GlobGetDeviceInfoReq               ((unsigned char)0x02)
#define LX_OID_GlobGetDeviceInfoConf              ((unsigned char)0x03)
#define LX_OID_GlobTestGroupIdReq                 ((unsigned char)0x04)
#define LX_OID_GlobTestGroupIdConf                ((unsigned char)0x05)
#define LX_OID_GlobTestObjectIdReq                ((unsigned char)0x06)
#define LX_OID_GlobTestObjectIdConf               ((unsigned char)0x07)
#define LX_OID_GlobMcReadReq                      ((unsigned char)0x08)
#define LX_OID_GlobMcReadConf                     ((unsigned char)0x09)
#define LX_OID_GlobMcWriteReq                     ((unsigned char)0x0a)
#define LX_OID_GlobObjectBufferOverFlowInd        ((unsigned char)0x0d)
#define LX_OID_GlobResetTimeReq                   ((unsigned char)0x0e)

#define LX_OID_GlobPortsDefineOutputsReq          ((unsigned char)0x10)
#define LX_OID_GlobPortsDefineOutputsConf         ((unsigned char)0x11)
#define LX_OID_GlobPortsWriteReq                  ((unsigned char)0x12)
#define LX_OID_GlobPortsWriteConf                 ((unsigned char)0x13)
#define LX_OID_GlobPortsReadReq                   ((unsigned char)0x14)
#define LX_OID_GlobPortsReadConf                  ((unsigned char)0x15)
#define LX_OID_GlobReadSerialNumberReq            ((unsigned char)0x16)
#define LX_OID_GlobReadSerialNumberConf           ((unsigned char)0x17)

#define LX_OID_GlobGeneralErrorInd                ((unsigned char)0xff)


/********* typedef **************************/

/* -- Driver -------------------------------*/
typedef char tLX_ShortFirmwareName[9];
typedef char tLX_SpecialFirmware[14];
typedef char tLX_ErrorString[81];
typedef char tLX_GeneralErrorString[252];
typedef char tLX_ShortBoardName[9];
typedef char tLX_LongBoardName[41];
typedef char tLX_BoardCode[9];
typedef char tLX_FirmwareCode[9];
typedef char tLX_VersionCode[6];

typedef char tLX_ManufacturerInformation[11];
typedef char tLX_ProductName[11];
typedef char tLX_SerialNumber[7];
typedef char tLX_HardwareVersion[6];
typedef char tLX_ProductionDate[5];
typedef char tLX_SNBoardCode[6];


#ifndef mLX_ObjHead
  #define mLX_ObjHead \
    unsigned char ObjLen;\
    unsigned char GroupID;\
    unsigned char ObjectID
#endif

typedef struct
{
  mLX_ObjHead;
  unsigned char         BoardNumber;
  tLX_ShortFirmwareName ShortFirmwareName;
  tLX_SpecialFirmware   SpecialFirmware;
} tLX_DriverOpenReq;
/*--------------------------*/

typedef struct
{
  mLX_ObjHead;
  tLX_ErrorString ErrorString;
} tLX_DriverOpenConf;
/*--------------------------*/

typedef struct
{
  mLX_ObjHead;
} tLX_DriverCloseReq;
/*--------------------------*/

typedef struct
{
  mLX_ObjHead;
} tLX_DriverGetInterfaceInfoReq;
/*--------------------------*/

typedef struct
{
  mLX_ObjHead;
  unsigned char      InterfaceType;
  unsigned long      Address;
  unsigned char      PortNumber;
  tLX_BoardCode      BoardCode;
  tLX_ShortBoardName ShortBoardName;
  tLX_LongBoardName  LongBoardName;
} tLX_DriverGetInterfaceInfoConf;
/*--------------------------*/

/* -- DirectAccess -------------------------*/

typedef struct
{
  mLX_ObjHead;
  unsigned char  AccessDestination;
  unsigned short Address;
  unsigned char  AccessType;
  unsigned char  Quantity;
} tLX_DirectAccessReadReq;
/*--------------------------*/

typedef struct
{
  mLX_ObjHead;
  unsigned char  AccessDestination;
  unsigned short Address;
  unsigned char  AccessType;
  unsigned char  Quantity;
  unsigned char  Result;
  unsigned char  Value[246];
} tLX_DirectAccessReadConf;
/*--------------------------*/

typedef struct
{
  mLX_ObjHead;
  unsigned char  AccessDestination;
  unsigned short Address;
  unsigned char  AccessType;
  unsigned char  Quantity;
  unsigned char  notused;
  unsigned char  Values[246];
} tLX_DirectAccessWriteReq;
/*--------------------------*/
typedef struct
{
  mLX_ObjHead;
  unsigned char  AccessDestination;
  unsigned short Address;
  unsigned char  AccessType;
  unsigned char  Quantity;
  unsigned char  Result;
} tLX_DirectAccessWriteConf;
/*--------------------------*/


/* -- Glob ---------------------------------*/

typedef struct
{
  mLX_ObjHead;
} tLX_GlobGetDeviceInfoReq;
/*--------------------------*/

typedef struct
{
  mLX_ObjHead;
  tLX_FirmwareCode      FirmwareCode;
  tLX_ShortFirmwareName ShortFirmwareName;
  tLX_VersionCode       VersionCode;
  tLX_LongBoardName     LongFirmwareName;
} tLX_GlobGetDeviceInfoConf;
/*--------------------------*/

typedef struct
{
  mLX_ObjHead;
  unsigned char TestGroupId;
} tLX_GlobTestGroupIdReq;
/*--------------------------*/

typedef struct
{
  mLX_ObjHead;
  unsigned char TestGroupId;
  unsigned char Error;
} tLX_GlobTestGroupIdConf;
/*--------------------------*/

typedef struct
{
  mLX_ObjHead;
  unsigned char TestGroupId;
  unsigned char TestObjectId;
} tLX_GlobTestObjectIdReq;
/*--------------------------*/
typedef struct
{
  mLX_ObjHead;
  unsigned char TestGroupId;
  unsigned char TestObjectId;
  unsigned char Error;
} tLX_GlobTestObjectIdConf;
/*--------------------------*/

typedef struct
{
  mLX_ObjHead;
  unsigned char Quantity;
  unsigned long Address;
} tLX_GlobMcReadReq;
/*--------------------------*/

typedef struct
{
  mLX_ObjHead;
  unsigned char Quantity;
  unsigned long Address;
  unsigned char Values[246];
} tLX_GlobMcReadConf;
/*--------------------------*/

typedef struct
{
  mLX_ObjHead;
  unsigned char Quantity;
  unsigned long Address;
  unsigned char Values[246];
} tLX_GlobMcWriteReq;
/*--------------------------*/

typedef struct
{
  mLX_ObjHead;
  unsigned char BufferType;
} tLX_GlobObjectBufferOverFlowInd;
/*--------------------------*/

typedef struct
{
  mLX_ObjHead;
} tLX_GlobResetTimeReq;
/*--------------------------*/


#define LX_GlobPortsMaxByteQuantity 32
typedef struct
{
  unsigned char Mask; // any 1-bit defines to use the accordingly Value-bit
  unsigned char Value;
} tLX_GlobPortsMaskAndValue;

typedef struct
{
  mLX_ObjHead;
  unsigned char ByteQuantity;  // e.g. 1 = one 8 bit port block
  unsigned char ByteAddress;   // e.g. 0 = 1st available 8 bit port block
  tLX_GlobPortsMaskAndValue MaskAndValue[LX_GlobPortsMaxByteQuantity];
  // depending on the Mask any Value-1-bit defines the port as output
} tLX_GlobPortsDefineOutputsReq;
typedef struct
{
  mLX_ObjHead;
  unsigned char ByteQuantity;
  unsigned char ByteAddress;
  tLX_GlobPortsMaskAndValue MaskAndValue[LX_GlobPortsMaxByteQuantity];
} tLX_GlobPortsDefineOutputsConf;
// if a port is not available,
//   ByteQuantity is reduced and/or the Mask-bit is set on 0.
// if a port definition as input/output is not possible,
//   the Mask-bit is as requested and
//   the Value-bit is set on the possible value.
/*--------------------------*/

typedef struct
{
  mLX_ObjHead;
  unsigned char ByteQuantity;
  unsigned char ByteAddress;
  tLX_GlobPortsMaskAndValue MaskAndValue[LX_GlobPortsMaxByteQuantity];
} tLX_GlobPortsWriteReq;
typedef struct
{
  mLX_ObjHead;
  unsigned char ByteQuantity;
  unsigned char ByteAddress;
  tLX_GlobPortsMaskAndValue MaskAndValue[LX_GlobPortsMaxByteQuantity];
} tLX_GlobPortsWriteConf;
/*--------------------------*/

typedef struct
{
  mLX_ObjHead;
  unsigned char ByteQuantity;
  unsigned char ByteAddress;
} tLX_GlobPortsReadReq;
typedef struct
{
  mLX_ObjHead;
  unsigned char ByteQuantity;
  unsigned char ByteAddress;
  unsigned char Value[LX_GlobPortsMaxByteQuantity];
} tLX_GlobPortsReadConf;
/*--------------------------*/


typedef struct
{
  mLX_ObjHead;
} tLX_GlobReadSerialNumberReq;
typedef struct
{
  mLX_ObjHead;
  unsigned char FlashDeviceID;
  tLX_ManufacturerInformation ManufacturerInformation;
  tLX_ProductName ProductName;
  tLX_SerialNumber SerialNumber;
  tLX_HardwareVersion HardwareVersion;
  tLX_ProductionDate ProductionDate;
  tLX_SNBoardCode BoardCode;
} tLX_GlobReadSerialNumberConf;
/*--------------------------*/


typedef struct
{
  mLX_ObjHead;
  tLX_GeneralErrorString ErrorString;
} tLX_GlobGeneralErrorInd;
/*--------------------------*/


#endif
